@@ -26,7 +26,7 @@
 */
 
 /** @file
- * @brief Class @ref Magnum::Math::Vector
+ * @brief Class @ref Magnum::Math::Vector, function @ref Magnum::Math::dot(), @ref Magnum::Math::angle()
  */
 
 #include <cmath>
@@ -42,10 +42,6 @@
 #include "Magnum/Math/BoolVector.h"
 #include "Magnum/Math/TypeTraits.h"
 
-#ifdef MAGNUM_BUILD_DEPRECATED
-#include <Corrade/Utility/Macros.h>
-#endif
-
 namespace Magnum { namespace Math {
 
 namespace Implementation {
@@ -148,26 +144,6 @@ template<std::size_t size, class T> class Vector {
             return padInternal<otherSize>(typename Implementation::GenerateSequence<size>::Type(), a, value);
         }
 
-        #ifdef MAGNUM_BUILD_DEPRECATED
-        /**
-         * @brief @copybrief Magnum::Math::dot(const Vector<size, T>&, const Vector<size, T>&)
-         * @deprecated Use @ref Math::dot(const Vector<size, T>&, const Vector<size, T>&)
-         *      instead.
-         */
-        CORRADE_DEPRECATED("use Math::dot() instead") static T dot(const Vector<size, T>& a, const Vector<size, T>& b) {
-            return Math::dot(a, b);
-        }
-
-        /**
-         * @brief @copybrief Magnum::Math::angle(const Vector<size, T>&, const Vector<size, T>&)
-         * @deprecated Use @ref Math::angle(const Vector<size, T>&, const Vector<size, T>&)
-         *      instead.
-         */
-        CORRADE_DEPRECATED("use Math::angle() instead") static Rad<T> angle(const Vector<size, T>& normalizedA, const Vector<size, T>& normalizedB) {
-            return Math::angle(normalizedA, normalizedB);
-        }
-        #endif
-
         /**
          * @brief Default constructor
          *
@@ -1099,7 +1075,7 @@ operator/(const Vector<size, Integral>& a, const Vector<size, FloatingPoint>& b)
     return copy /= b;
 }
 
-/** @debugoperator{Magnum::Math::Vector} */
+/** @debugoperator{Vector} */
 template<std::size_t size, class T> Corrade::Utility::Debug& operator<<(Corrade::Utility::Debug& debug, const Vector<size, T>& value) {
     debug << "Vector(" << Corrade::Utility::Debug::nospace;
     for(std::size_t i = 0; i != size; ++i) {
